{% for import_line in import_lines %}
{{ import_line }}
{% endfor %}


from dspygen.rdddy.base_actor import BaseActor

class {{ actor_name }}(BaseActor):
    def __init__(self, actor_system, actor_id=None):
        super().__init__(actor_system, actor_id)

    {% for message in messages %}
    async def handle_{{ message.type|lower }}(self, message: {{ message.class_name }}):
        """
        Handle {{ message.type }} event.
        {% if message.description %}# {{ message.description }}{% endif %}
        """
        # Logic to handle {{ message.type }} event
        pass
    {% endfor %}
