def get_current_weather(location, temperature_unit) do """ Get the current weather for a given location in the specified format. :param location: The city and state, e.g., "San Francisco, CA". :param temperature_unit: The temperature unit to use, either "celsius" or "fahrenheit". :return: A string describing the current weather. """ IO.puts("Retrieving weather for #{location}, temperature unit: #{temperature_unit}") # TODO: API Call "#{location}, #{temperature_unit}" end def get_n_day_weather_forecast(location, temperature_unit, num_days) do """ Get an N-day weather forecast for a given location in the specified format. :param location: The city and state, e.g., "San Francisco, CA". :param temperature_unit: The temperature unit to use, either "celsius" or "fahrenheit". :param num_days: The number of days to forecast. :return: A string describing the weather forecast. """ IO.puts("Retrieving #{num_days} day weather forecast for #{location} in #{temperature_unit}") "#{location}, #{temperature_unit}, #{num_days}" end